/*

  Copyright 2004-2007 CandU

  This file is part of the Halogen demo.

  Halogen is free software; you can redistribute it and/or modify it under the 
  terms of the GNU General Public License as published by the Free Software 
  Foundation; either version 2 of the License, or (at your option) any later 
  version.

  Halogen is distributed in the hope that it will be useful, but WITHOUT ANY 
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE. See the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along with
  Halogen; if not, write to the Free Software Foundation, Inc., 51 Franklin 
  Street, Fifth Floor, Boston, MA 02110-1301, USA 

*/

#include <bladeps.h>
#include <syscall.h>

/* Graphics files */
#include "halogen.h"
#include "letters.h"
#include "candu.h"
#include "wave.h"

/* Music files */
#include "winlevel.h"

/* Constants */
#define TOP_LIMIT               0
#define BOTTOM_LIMIT            240

#define STATE_INTRO				0x00
#define STATE_SCREEN1			0x01
#define STATE_SCREEN1_CLOSING	0x02
#define STATE_SCREEN2_OPENING	0x03
#define STATE_SCREEN2_RUNNING	0x04

#define SHAPE_BOUNCE_NONE		0x00
#define SHAPE_BOUNCE_TOP		0x01
#define SHAPE_BOUNCE_BOTTOM		0x02
#define SHAPE_BOUNCE_LEFT		0x04
#define SHAPE_BOUNCE_RIGHT		0x08

#define NUM_SHAPES              12

/* Primitives buffer */
u_char primitiveBuffer[2][4000];

/* Shape struct for rects & stars */
typedef struct
{
	int cx;             /* Centre pos and radius */
	int cy;
	int r;
	int dr;
	int cdx;            /* Movement animation */
	int cdy;
	u_short angle;      /* Current rotation */
	short da;           /* Rotation animation */
	int bounce;         /* Bounce state */
}Shape;

/* Images & Polygons*/
GsIMAGE halogenImage;
GsSPRITE halogenSprite;

GsIMAGE canduImage;
GsSPRITE canduSprite;

GsIMAGE textImage;
GsTPOLY4 textPoly;

GsIMAGE waveImage;
GsSPRITE waveSprite;
GsTPOLY4 wavePoly;

GsTILE shutterTile;

/* Intro screen info */
int canduInt;
int canduDir;

/* Scrolling text info */
int textLen;

char textString[512];

int textPos;
int textScrollPos;
int sinPos;
int shutterSize;
char temp[32];

int xInstr;
int dxInstr;
u_char bkPos;
char dbk;

/* Shape screen info */
int uBackground;
int vBackground;
int uShapes;
int vShapes;

/* Functions */
void sprites_Init(void);
void textString_Init(void);
void textString_Draw(void);
void textString_Scroll(void);
void background_Draw(void);
void shape_Init(Shape *pSh);
void shape_Animate(Shape *pSh);
void shape_NewDir(Shape *pSh,int dx,int dy);
void square_Draw(Shape *pSq);
void star_Draw(Shape *pSt);
void poly_SetUV(void);
int poly_CheckBounds(void);

void shutters_Draw(void);
void instructions_Draw(void);

void main (void)
{
	int activeBuffer;
	u_long pad,lastPad;
	Shape shapes[NUM_SHAPES];
	int sh,state;

    /* Set up pad */
	PadInit();

	srand(124);

	strcpy(textString,"            ...WELCOME TO HALOGEN. THIS IS MY FIRST PSONE \
DEMO SO DONT EXPECT MIRACLES. MUSIC IS BORROWED FROM THE AMIGA VERSION OF \
DESERT STRIKE. GRFX AND CODE BY TJ, BUILT USING BLADE PSX LIBS AND HITMEN MOD \
PLAYER. HELLOS GO TO ULLA, BIG NIGE, LUIJI, GAVDAV, JEH AND MIKIE T MAN. \
ALSO A BIG HELLO TO STREAK AND ALL THE GUYS IN POLAND.    \
PRESS CROSS TO CONTINUE...");

    /* Initialise values for animation */
	bkPos=0;
	shutterSize=0;

	uBackground=128;
	vBackground=0;
	uShapes=128;
	vShapes=128;

    textPos=0;
	textScrollPos=0;
	sinPos=0;

	dbk=1;
	xInstr=460;
	dxInstr=130;

	canduInt=0;
	canduDir=1;

	sprites_Init();
	textString_Init();

    /* Set up graphice */
	SetVideoMode(MODE_AUTO);
	GsInitGraph(320,240,0,1,0);
	GsDefDispBuff(0,0,0,240);

	SsInit();

	state=STATE_INTRO;
	for(sh=0;sh<NUM_SHAPES;sh++)
	{
		shape_Init(&shapes[sh]);
	}

	while(1)
	{
		PollMod(0);

		activeBuffer=GsGetActiveBuff();

		GsSetWorkBase(&primitiveBuffer[activeBuffer][0]);

		GsSortClear(0,0,0);

		switch(state)
		{
		case STATE_INTRO:
            /* Render intro screen */
			canduInt+=canduDir;
			canduSprite.r=canduSprite.g=canduSprite.b=canduInt>128?128:canduInt;

            /* Fade in, then out */
			if(canduInt==256)
			{
				canduDir=-canduDir;
			}
			else if(canduInt==0)
			{
                /* We have faded in and out - move on to the next screen */
				InitMod(0,winlevel);
				state=STATE_SCREEN1;
			}

			GsSortSprite(&canduSprite);
			break;

		case STATE_SCREEN1:
		case STATE_SCREEN1_CLOSING:
            /* Fade in the halogen graphics */
			if(halogenSprite.r<128)
			{
				halogenSprite.r=halogenSprite.g=halogenSprite.b=halogenSprite.r+1;
			}

			GsSortSprite(&halogenSprite);
			textString_Draw();
			textString_Scroll();
			break;

		case STATE_SCREEN2_OPENING:
		case STATE_SCREEN2_RUNNING:
            /* Draw background from the tiles */
			background_Draw();
			
			if(state==STATE_SCREEN2_RUNNING)
			{
                /* Draw the shapes */
				for(sh=0;sh<NUM_SHAPES;sh++)
				{
					if(sh<NUM_SHAPES/2)
					{
						square_Draw(&shapes[sh]);
					}
					else
					{
						star_Draw(&shapes[sh]);
					}
				}
			}
			instructions_Draw();

			break;
		
		default:
			break;
		}

        /* Draw shutters */
		if(state==STATE_SCREEN1_CLOSING || state==STATE_SCREEN2_OPENING)
		{
			shutters_Draw();
		}

        /* Swap buffers and wait for vsync */
	    GsSwapDispBuff();

		GsDrawOT((u_long*)&primitiveBuffer[activeBuffer][0]);
		WaitVSync();

        /* Animate the screens by updating animate values */
		switch(state)
		{
		case STATE_SCREEN2_RUNNING:
			bkPos+=dbk;
			bkPos%=64;

			for(sh=0;sh<NUM_SHAPES;sh++)
			{
				shape_Animate(&shapes[sh]);
			}
			break;

		case STATE_SCREEN2_OPENING:
			bkPos+=dbk;
			bkPos%=64;
			shutterSize-=2;
			if(shutterSize<0)
			{
				state=STATE_SCREEN2_RUNNING;
			}
			break;

		case STATE_SCREEN1_CLOSING:
			shutterSize+=2;
			if(shutterSize>(BOTTOM_LIMIT/2))
			{
				state=STATE_SCREEN2_OPENING;
			}
			break;
		}

        /* Handle pad input from the user */
	    pad=PadRead();

		if(pad!=lastPad && (pad&PAD_CROSS)!=0)
		{
			switch(state)
			{
			case STATE_INTRO:
				InitMod(0,winlevel);
				state=STATE_SCREEN1;
				break;

			case STATE_SCREEN1:
				state=STATE_SCREEN1_CLOSING;
				break;

			case STATE_SCREEN2_RUNNING:
				vBackground=128-vBackground;
				vShapes=128-vShapes;
				waveSprite.v=waveImage.py+vBackground;
				break;

			}
				
		}
		else if(pad!=lastPad && (pad&PAD_SQUARE)!=0)
		{
			if(state==STATE_SCREEN2_RUNNING)
			{
				dbk=-dbk;
			}
		}
		else if(pad!=lastPad && (pad&PAD_CIRCLE)!=0)
		{
			if(state==STATE_SCREEN2_RUNNING)
			{
				dxInstr=dxInstr==130?320:130;
			}
		}
		else if(pad!=lastPad && (pad&PAD_TRIANGLE)!=0)
		{
			if(state==STATE_SCREEN2_RUNNING)
			{
				uBackground=128-uBackground;
				uShapes=128-uShapes;
				waveSprite.u=waveImage.px+uBackground;
			}
		}

		lastPad=pad;

	}
}

/* Initialises the sprites used in the demo */
void sprites_Init(void)
{
	LoadTim((TIMHEADER*)wave);
	GsGetTimInfo((TIMHEADER*)wave,&waveImage);

	waveSprite.r = waveSprite.g = waveSprite.b = 128;
	waveSprite.x = 0;
	waveSprite.y = 64;
	waveSprite.w = 128;
	waveSprite.h = 128;
	waveSprite.u = waveImage.px+uBackground;
	waveSprite.v = waveImage.py+vBackground;
	waveSprite.clutid = waveImage.clutid;
	waveSprite.tpage = waveImage.tpage;
	waveSprite.scalex = ONE;
	waveSprite.scaley = ONE;
	waveSprite.rotate = ONE;

	wavePoly.r = wavePoly.g = wavePoly.b = 128;

	wavePoly.clutid = waveImage.clutid;
	wavePoly.tpage = waveImage.tpage;
	wavePoly.x0=100;	wavePoly.y0=100;	wavePoly.u0=0;	wavePoly.v0=0;
	wavePoly.x1=150;	wavePoly.y1=100;	wavePoly.u1=0;	wavePoly.v1=0;
	wavePoly.x2=100;	wavePoly.y2=150;	wavePoly.u2=0;	wavePoly.v2=0;
	wavePoly.x3=150;	wavePoly.y3=150;	wavePoly.u3=0;	wavePoly.v3=0;

	LoadTim((TIMHEADER*)letters);
	GsGetTimInfo((TIMHEADER*)letters,&textImage);

	textPoly.r = textPoly.g = textPoly.b = 128;

	textPoly.clutid = textImage.clutid;
	textPoly.tpage = textImage.tpage;
	textPoly.x0=0;	textPoly.y0=0;	textPoly.u0=0;	textPoly.v0=0;
	textPoly.x1=0;	textPoly.y1=0;	textPoly.u1=0;	textPoly.v1=0;
	textPoly.x2=0;	textPoly.y2=0;	textPoly.u2=0;	textPoly.v2=0;
	textPoly.x3=0;	textPoly.y3=0;	textPoly.u3=0;	textPoly.v3=0;

	LoadTim((TIMHEADER*)halogen);
	GsGetTimInfo((TIMHEADER*)halogen,&halogenImage);

	halogenSprite.r = halogenSprite.g = halogenSprite.b = 0;
	halogenSprite.x = (320-halogenImage.pw)/2;
	halogenSprite.y = 40;
	halogenSprite.w = halogenImage.pw;
	halogenSprite.h = halogenImage.ph;
	halogenSprite.u = halogenImage.px;
	halogenSprite.v = halogenImage.py;
	halogenSprite.clutid = halogenImage.clutid;
	halogenSprite.tpage = halogenImage.tpage;
	halogenSprite.scalex = ONE;
	halogenSprite.scaley = ONE;
	halogenSprite.rotate = ONE;

	LoadTim((TIMHEADER*)candu);
	GsGetTimInfo((TIMHEADER*)candu,&canduImage);

	canduSprite.r = canduSprite.g = canduSprite.b = 0;
	canduSprite.x = (320-canduImage.pw)/2;
	canduSprite.y = (240-canduImage.ph)/2;;
	canduSprite.w = canduImage.pw;
	canduSprite.h = canduImage.ph;
	canduSprite.u = canduImage.px;
	canduSprite.v = canduImage.py;
	canduSprite.clutid = canduImage.clutid;
	canduSprite.tpage = canduImage.tpage;
	canduSprite.scalex = ONE;
	canduSprite.scaley = ONE;
	canduSprite.rotate = ONE;

}

/* Initialises the text string - the string is adjusted in place from ascii */
/* to the indexes of the glyph used for each character */
void textString_Init(void)
{
	int ch;
	textLen=strlen(textString);

	printf("debug");
	for(ch=0;ch<textLen;ch++)
	{
		if(textString[ch]>='A' && textString[ch]<='Z')
		{
			textString[ch]-='A';
		}
		else if(textString[ch]>='0' && textString[ch]<='9')
		{
			textString[ch]-='0';
			textString[ch]+=26;
		}
		else if(textString[ch]=='.')
		{
			textString[ch]=36;
		}
		else if(textString[ch]==',')
		{
			textString[ch]=37;
		}
		else
		{
			textString[ch]=38;
		}

	}
}

/* Draw the glyphs for the string, starting at the first on-screen glyph and */
/* ending once off screen to the right */
void textString_Draw(void)
{
	int ch,rch,cch,xVal,yVal,xPos,sinA;

	rch=textPos;

	xPos=-textScrollPos;

	sinA=sin(sinPos*16)/256;

	for(ch=0;ch<13;ch++)
	{
		cch=textString[rch];
		xVal=(cch%7)*27;
		yVal=(cch/7)*41;

		textPoly.x0=xPos;		textPoly.y0=160+sinA;	textPoly.u0=textImage.px+xVal;		textPoly.v0=textImage.py+yVal;
		textPoly.x1=xPos+27;	textPoly.y1=160+sinA;	textPoly.u1=textImage.px+xVal+27;	textPoly.v1=textImage.py+yVal;
		textPoly.x2=xPos;		textPoly.y2=201+sinA;	textPoly.u2=textImage.px+xVal;		textPoly.v2=textImage.py+yVal+41;
		textPoly.x3=xPos+27;	textPoly.y3=201+sinA;	textPoly.u3=textImage.px+xVal+27;	textPoly.v3=textImage.py+yVal+41;

		GsSortTPoly4(&textPoly);

		rch++;
		rch%=textLen;

		xPos+=27;
	}
}

/* Update animation values for the scrolling text */
void textString_Scroll(void)
{
	if(textScrollPos<26)
	{
		textScrollPos++;
	}
	else
	{
		textScrollPos=0;
		textPos++;
		textPos%=textLen;
	}

	sinPos=(sinPos+1)%256;
}

/* Draw the background for the shape screen from the tiles */
void background_Draw(void)
{
	int xPos;

	xPos=-bkPos;

	waveSprite.w = 128;

	while(xPos<320)
	{
		waveSprite.x=xPos;
		if(xPos+waveSprite.w>320)
		{
			waveSprite.w=320-xPos;
		}

		waveSprite.y=0;
		GsSortSprite(&waveSprite);
		waveSprite.y=128;
		GsSortSprite(&waveSprite);

		xPos+=waveSprite.w;
	}
}

/* Draw a square */
void square_Draw(Shape *pSq)
{
	int cx,cy,r;
	int ds,dc;
	u_short angle;

	cx=pSq->cx/256;
	cy=pSq->cy/256;
	r=pSq->r;
	angle=pSq->angle;

	ds=(((int)sin(angle))*r)/4096;
	dc=(((int)cos(angle))*r)/4096;

	wavePoly.x0=cx+ds;	wavePoly.y0=cy-dc;
	wavePoly.x1=cx+dc;	wavePoly.y1=cy+ds;
	wavePoly.x2=cx-dc;	wavePoly.y2=cy-ds;
	wavePoly.x3=cx-ds;	wavePoly.y3=cy+dc;

	pSq->bounce=poly_CheckBounds();
	poly_SetUV();
	
	GsSortTPoly4(&wavePoly);
}

/* Calculates the texture coordinates for a quad to ensure it lines up with */
/* the background texture */
void poly_SetUV(void)
{
	int u0,v0,u1,v1,u2,v2,u3,v3;

	u0=(wavePoly.x0+bkPos)%64;
	v0=wavePoly.y0%64;
	u1=u0+wavePoly.x1-wavePoly.x0;
	v1=v0+wavePoly.y1-wavePoly.y0;
	u2=u0+wavePoly.x2-wavePoly.x0;
	v2=v0+wavePoly.y2-wavePoly.y0;
	u3=u0+wavePoly.x3-wavePoly.x0;
	v3=v0+wavePoly.y3-wavePoly.y0;

	while(u0<0 || u1<0 || u2<0 || u3<0)
	{
		u0+=64;		u1+=64;		u2+=64;		u3+=64;
	}

	while(u0>128 || u1>128 || u2>128 || u3>128)
	{
		u0-=64;		u1-=64;		u2-=64;		u3-=64;
	}

	while(v0<0 || v1<0 || v2<0 || v3<0)
	{
		v0+=64;		v1+=64;		v2+=64;		v3+=64;
	}

	while(v0>128 || v1>128 || v2>128 || v3>128)
	{
		v0-=64;		v1-=64;		v2-=64;		v3-=64;
	}

	wavePoly.u0=uShapes+u0;	wavePoly.v0=vShapes+v0;
	wavePoly.u1=uShapes+u1;	wavePoly.v1=vShapes+v1;
	wavePoly.u2=uShapes+u2;	wavePoly.v2=vShapes+v2;
	wavePoly.u3=uShapes+u3;	wavePoly.v3=vShapes+v3;
}

/* Creates a new destination size and angle used once we have collided */
void shape_SizeAngle(Shape *pSh)
{
	pSh->dr=rand()%32;
	pSh->da=(rand()%40)-20;
}

/* Initialise a new shape */
void shape_Init(Shape *pSh)
{
	pSh->cx=((rand()%300)+10)*256;
	pSh->cy=((rand()%(BOTTOM_LIMIT-TOP_LIMIT-20))+10)*256;

	shape_NewDir(pSh,0,0);
	pSh->r=0;
	pSh->angle=0;

	pSh->bounce=SHAPE_BOUNCE_NONE;
	shape_SizeAngle(pSh);
}

/* Creates a new direction used once we have collided */
void shape_NewDir(Shape *pSh,int dx,int dy)
{
	pSh->cdx=(rand()%384); 
	if(pSh->cdx<192)
	{
		pSh->cdx-=256;
	}
	else
	{
		pSh->cdx-=128;
	}
	
	pSh->cdy=(rand()%512); 
	if(pSh->cdy<384)
	{
		pSh->cdy-=256;
	}
	else
	{
		pSh->cdy-=128;
	}

	if((dx*pSh->cdx)<0)
	{
		pSh->cdx=-pSh->cdx;
	}

	if((dy*pSh->cdy)<0)
	{
		pSh->cdy=-pSh->cdy;
	}
}

/* Animates a shape and checks for collisions */
void shape_Animate(Shape *pSh)
{
	int cx,cy;
	pSh->cx+=pSh->cdx;
	pSh->cy+=pSh->cdy;

	if((pSh->bounce&SHAPE_BOUNCE_TOP)!=0)
	{
		shape_NewDir(pSh,pSh->cdx,1);
		shape_SizeAngle(pSh);
	}
	else if((pSh->bounce&SHAPE_BOUNCE_BOTTOM)!=0)
	{
		shape_NewDir(pSh,pSh->cdx,-1);
		shape_SizeAngle(pSh);
	}
	else if((pSh->bounce&SHAPE_BOUNCE_LEFT)!=0)
	{
		shape_NewDir(pSh,1,pSh->cdy);
		shape_SizeAngle(pSh);
	}
	else if((pSh->bounce&SHAPE_BOUNCE_RIGHT)!=0)
	{
		shape_NewDir(pSh,-1,pSh->cdy);
		shape_SizeAngle(pSh);
	}

	cx=pSh->cx/256;
	cy=pSh->cy/256;
	if((pSh->r<pSh->dr) && (cx>pSh->dr) && (320-cx>pSh->dr) &&
		(cy-TOP_LIMIT>pSh->dr) && (BOTTOM_LIMIT-cy>pSh->dr))
	{
		pSh->r++;
	}
	else if(pSh->r>pSh->dr)
	{
		pSh->r--;
	}

	pSh->angle+=pSh->da;
	pSh->angle%=4096;
}

/* Draws a star: stars consist of 3 quads as follows                          */
/*                                                                            */
/*     /\       /\        __________     ____/\____                           */
/*    / /       \ \       \        /     \        /                           */
/*   / /    +    \ \   +   \______/   =   \  __  /                            */
/*  /_/           \_\                     /_/  \_\                            */
/* //               \\                   //      \\                           */
/*                                                                            */
void star_Draw(Shape *pSt)
{
	int sins[5];
	int coss[5];
	int cx,cy,p,angle;
	int rB;

	pSt->bounce=SHAPE_BOUNCE_NONE;

	cx=pSt->cx/256;
	cy=pSt->cy/256;

	for(p=0;p<5;p++)
	{
		angle=pSt->angle+(4096*p/5);

		sins[p]=(int)sin(angle);
		coss[p]=(int)cos(angle);
	}

	rB=cos(819)*pSt->r;
	// Draw 1st quad
	wavePoly.x0=cx+(sins[0]*pSt->r)/0x1000;
	wavePoly.y0=cy+(coss[0]*pSt->r)/0x1000;
	wavePoly.x1=cx+(sins[2]*pSt->r)/0x1000;
	wavePoly.y1=cy+(coss[2]*pSt->r)/0x1000;
	wavePoly.x2=cx-(sins[2]*rB)/0x1000000;
	wavePoly.y2=cy-(coss[2]*rB)/0x1000000;
	wavePoly.x3=cx-(sins[0]*rB)/0x1000000;
	wavePoly.y3=cy-(coss[0]*rB)/0x1000000;

	pSt->bounce|=poly_CheckBounds();
	poly_SetUV();

	GsSortTPoly4(&wavePoly);

	// Draw 2nd quad

	wavePoly.x0=cx+(sins[4]*pSt->r)/0x1000;
	wavePoly.y0=cy+(coss[4]*pSt->r)/0x1000;
	wavePoly.x1=cx+(sins[1]*pSt->r)/0x1000;
	wavePoly.y1=cy+(coss[1]*pSt->r)/0x1000;
	wavePoly.x2=cx-(sins[1]*rB)/0x1000000;
	wavePoly.y2=cy-(coss[1]*rB)/0x1000000;
	wavePoly.x3=cx-(sins[4]*rB)/0x1000000;
	wavePoly.y3=cy-(coss[4]*rB)/0x1000000;

	pSt->bounce|=poly_CheckBounds();
	poly_SetUV();

	GsSortTPoly4(&wavePoly);


	// Draw 3rd quad

	wavePoly.x0=cx+(sins[0]*pSt->r)/0x1000;
	wavePoly.y0=cy+(coss[0]*pSt->r)/0x1000;
	wavePoly.x1=cx+(sins[3]*pSt->r)/0x1000;
	wavePoly.y1=cy+(coss[3]*pSt->r)/0x1000;
	wavePoly.x2=cx-(sins[3]*rB)/0x1000000;
	wavePoly.y2=cy-(coss[3]*rB)/0x1000000;
	wavePoly.x3=cx-(sins[0]*rB)/0x1000000;
	wavePoly.y3=cy-(coss[0]*rB)/0x1000000;

	pSt->bounce|=poly_CheckBounds();
	poly_SetUV();

	GsSortTPoly4(&wavePoly);
}

/* Draw shutters at top and bottom over the other graphics */
void shutters_Draw(void)
{
	shutterTile.r=shutterTile.g=shutterTile.b=0;

	shutterTile.x=0;
	shutterTile.y=0;
	shutterTile.w=320;
	shutterTile.h=shutterSize;
	shutterTile.trans=0;

	GsSortTile(&shutterTile);

	shutterTile.y=BOTTOM_LIMIT-shutterSize;

	GsSortTile(&shutterTile);
}

void instructions_Draw(void)
{
	FntColor(0,0,0,0);

	FntPrint(0,xInstr,160,"INSTRUCTIONS");
	FntPrint(0,xInstr,176,"CROSS  : COLOURS");
	FntPrint(0,xInstr,188,"SQUARE : DIRECTION");
	FntPrint(0,xInstr,200,"TRIANG : WAVE/STRAIGHT");
	FntPrint(0,xInstr,212,"CIRCLE : SHOW/HIDE");

	if(xInstr>dxInstr)
	{
		xInstr-=3;
	}

	if(xInstr<dxInstr)
	{
		xInstr+=3;
	}
}

/* Check for collisions between a shape and the edge */
int poly_CheckBounds(void)
{
	int bounce=SHAPE_BOUNCE_NONE;

	if(wavePoly.x0<0 || wavePoly.x1<0 || wavePoly.x2<0 || wavePoly.x3<0)
	{
		bounce+=SHAPE_BOUNCE_LEFT;
	}

	if(wavePoly.x0>320 || wavePoly.x1>320 || wavePoly.x2>320 || wavePoly.x3>320)
	{
		bounce+=SHAPE_BOUNCE_RIGHT;
	}

	if(wavePoly.y0<TOP_LIMIT || wavePoly.y1<TOP_LIMIT || wavePoly.y2<TOP_LIMIT || wavePoly.y3<TOP_LIMIT)
	{
		bounce+=SHAPE_BOUNCE_TOP;
	}

	if(wavePoly.y0>BOTTOM_LIMIT || wavePoly.y1>BOTTOM_LIMIT || wavePoly.y2>BOTTOM_LIMIT || wavePoly.y3>BOTTOM_LIMIT)
	{
		bounce+=SHAPE_BOUNCE_BOTTOM;
	}

	return(bounce);
}

